///////////////////////////////////////////////////////////////////////////////
/// @file   OpticalAndStereo.h
///////////////////////////////////////////////////////////////////////////////

#ifndef CORNER_H_INCLUDED_
#define CORNER_H_INCLUDED_

#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include <src/ImageProcessingAPI/base/Array.h>
#include <src/ImageProcessingAPI/base/Common.h>
#include <src/vision_rtm/VideoFrameSyncComponentBase.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <float.h>
#include <unistd.h>
#include <pthread.h>
#include <iostream>
#include <sstream>
#include <rtvlib.h>

#ifndef PI
#define PI (3.1415926535)
#endif

//Optical Flow
#define OF_SIZE 	(16)
#define OF_MAG 		(1)
#define OF_SAMPLE 	(OF_MAG - 1)
#define OF_DISP_MAG 	(1)
#define OF_HSEARCH 	(16)
#define OF_VSEARCH 	(16)
#define OF_GAP 		(0)
#define OF_PITCH 	(OF_SIZE + OF_GAP)
#define CENTER_TO_REF 	((OF_SIZE / 2) * OF_MAG)
#define REF_TO_SER_H 	((OF_HSEARCH / 2) * OF_MAG)
#define REF_TO_SER_V 	((OF_VSEARCH / 2) * OF_MAG)
#define DISP_CENTER_H 	(320)
#define DISP_CENTER_V 	(240)
#define OFNH 		(74)
#define OFNV 		(54)

typedef struct {
    int ch, cv; 	/* 計測中心座標 = 動きベクトルの始点 */
    int eh, ev; 	/* 動きベクトルの終点 */
    int rh, rv; 	/* 参照ブロックの左上座標 */
    int sh, sv; 	/* 探索ブロックの左上座標 */
    int dh, dv, mdist; /* 動きベクトルと最小相関値 */
    int mdistc;
} Flow_data_t;

#define	FILT_OFF_H	(3)
#define	FILT_OFF_V	(3)
#define	FILT_DIV	(240)
#define	IMG_START_H	(0)
#define	IMG_START_V	(0)
#define	IMG_HSIZE_H	(320)
#define	IMG_HSIZE_V	(240)
#define	FILT_MARGIN_H	(20)
#define	FILT_MARGIN_V	(20)

#define	H_SEARCH	(4)
#define	V_SEARCH	(5)
#define	H_PITCH		(8)
#define	V_PITCH		(10)

#define IMG_HSIZE (640)
#define IMG_VSIZE (480)
#define IMAGE_HMARGIN (30)
#define IMAGE_VMARGIN (30)

#define	MATCH_10_SEARCH_H	(6)
#define	MATCH_10_SEARCH_V	(6)

#define FEATURE_TEMP_CH (4)
#define FEATURE_TEMP_CV (4)
#define	FEATURE_SEARCH_H	(5)
#define	FEATURE_SEARCH_V	(5)

#define	WIN_H	(8)
#define	WIN_V	(8)
#define	WIN_HH	(WIN_H / 2)
#define	WIN_HV	(WIN_V / 2)

#define MAX_STEREO3D_RESULT (2000)

char MaskSobelGH[64] = {
    -1,  -1,  -1,   0,   1,   1,   1,   0,
    -3,  -4,  -5,   0,   5,   4,   3,   0,
    -5,  -9, -13,   0,  13,   9,   5,   0,
    -6, -12, -18,   0,  18,  12,   6,   0,
    -5,  -9, -13,   0,  13,   9,   5,   0,
    -3,  -4,  -5,   0,   5,   4,   3,   0,
    -1,  -1,  -1,   0,   1,   1,   1,   0,
    0,   0,   0,   0,   0,   0,   0,   0
};
  
char MaskSobelGV[64] = {
    -1,  -3,  -5,  -6,  -5,  -3,  -1,   0,
    -1,  -4,  -9, -12,  -9,  -4,  -1,   0,
    -1,  -5, -13, -18, -13,  -5,  -1,   0,
    0,   0,   0,   0,   0,   0,   0,   0,
    1,   5,  13,  18,  13,   5,   1,   0,
    1,   4,   9,  12,   9,   4,   1,   0,
    1,   3,   5,   6,   5,   3,   1,   0,
    0,   0,   0,   0,   0,   0,   0,   0
};

float	m_RP[12] = {
    558.042, 2.31129, 313.007, -50197.7,
    -0.934862, 557.418, 246.39, 1318.71,
    -0.000294, 0.004622, 0.999989, 0.426643
};
		
float	m_LP[12] = {
    559.634521, 0.0000000000, 312.406128, 0.0000000000,
    0.0000000000, 557.862061, 254.5111080, 0.0000000000,
    0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000
};

#define	EP1 (((m_LP[10] * m_LP[4] - m_LP[6] * m_LP[8]) * m_RP[7] + \
		  (m_LP[7] * m_LP[8] - m_LP[11] * m_LP[4]) * m_RP[6] + \
		  (m_LP[11] * m_LP[6] - m_LP[10] * m_LP[7]) * m_RP[4]) * m_RP[9] + \
		 ((m_LP[6] * m_LP[9] - m_LP[10] * m_LP[5]) * m_RP[7] + \
		  (m_LP[11] * m_LP[5] - m_LP[7] * m_LP[9]) * m_RP[6] + \
		  (m_LP[10] * m_LP[7] - m_LP[11] * m_LP[6]) * m_RP[5]) * m_RP[8] + \
		 (m_LP[5] * m_LP[8] - m_LP[4] * m_LP[9]) * m_RP[10] * m_RP[7] + \
		 (m_LP[4] * m_LP[9] - m_LP[5] * m_LP[8]) * m_RP[11] * m_RP[6] + \
		 ((m_LP[6] * m_LP[8] - m_LP[10] * m_LP[4]) * m_RP[11] + \
		  (m_LP[11] * m_LP[4] - m_LP[7] * m_LP[8]) * m_RP[10]) * m_RP[5] + \
		 ((m_LP[10] * m_LP[5] - m_LP[6] * m_LP[9]) * m_RP[11] + \
		  (m_LP[7] * m_LP[9] - m_LP[11] * m_LP[5]) * m_RP[10]) * m_RP[4])

#define	EQ1 (((m_LP[6] * m_LP[8] - m_LP[10] * m_LP[4]) * m_RP[3] + \
		  (m_LP[11] * m_LP[4] - m_LP[7] * m_LP[8]) * m_RP[2] + \
		  (m_LP[10] * m_LP[7] - m_LP[11] * m_LP[6]) * m_RP[0]) * m_RP[9] + \
		 ((m_LP[10] * m_LP[5] - m_LP[6] * m_LP[9]) * m_RP[3] + \
		  (m_LP[7] * m_LP[9] - m_LP[11] * m_LP[5]) * m_RP[2] + \
		  (m_LP[11] * m_LP[6] - m_LP[10] * m_LP[7]) * m_RP[1]) * m_RP[8] + \
		 (m_LP[4] * m_LP[9] - m_LP[5] * m_LP[8]) * m_RP[10] * m_RP[3] + \
		 (m_LP[5] * m_LP[8] - m_LP[4] * m_LP[9]) * m_RP[11] * m_RP[2] + \
		 ((m_LP[10] * m_LP[4] - m_LP[6] * m_LP[8]) * m_RP[1] + \
		  (m_LP[6] * m_LP[9] - m_LP[10] * m_LP[5]) * m_RP[0]) * m_RP[11] + \
		 ((m_LP[7] * m_LP[8] - m_LP[11] * m_LP[4]) * m_RP[1] + \
		  (m_LP[11] * m_LP[5] - m_LP[7] * m_LP[9]) * m_RP[0]) * m_RP[10])

#define ER1 (((m_LP[4] * m_LP[9] - m_LP[5] * m_LP[8]) * m_RP[2] + \
		  (m_LP[6] * m_LP[8] - m_LP[10] * m_LP[4]) * m_RP[1] + \
		  (m_LP[10] * m_LP[5] - m_LP[6] * m_LP[9]) * m_RP[0]) * m_RP[7] + \
		 ((m_LP[5] * m_LP[8] - m_LP[4] * m_LP[9]) * m_RP[3] + \
		  (m_LP[11] * m_LP[4] - m_LP[7] * m_LP[8]) * m_RP[1] + \
		  (m_LP[7] * m_LP[9] - m_LP[11] * m_LP[5]) * m_RP[0]) * m_RP[6] + \
		 ((m_LP[10] * m_LP[4] - m_LP[6] * m_LP[8]) * m_RP[3] + \
		  (m_LP[7] * m_LP[8] - m_LP[11] * m_LP[4]) * m_RP[2] + \
		  (m_LP[11] * m_LP[6] - m_LP[10] * m_LP[7]) * m_RP[0]) * m_RP[5] + \
		 ((m_LP[6] * m_LP[9] - m_LP[10] * m_LP[5]) * m_RP[3] + \
		  (m_LP[11] * m_LP[5] - m_LP[7] * m_LP[9]) * m_RP[2] + \
		  (m_LP[10] * m_LP[7] - m_LP[11] * m_LP[6]) * m_RP[1]) * m_RP[4])

#define EP2 (((m_LP[0] * m_LP[6] - m_LP[2] * m_LP[4]) * m_RP[7] + \
		  (m_LP[3] * m_LP[4] - m_LP[0] * m_LP[7]) * m_RP[6] + \
		  (m_LP[2] * m_LP[7] - m_LP[3] * m_LP[6]) * m_RP[4]) * m_RP[9] + \
		 ((m_LP[2] * m_LP[5] - m_LP[1] * m_LP[6]) * m_RP[7] + \
		  (m_LP[1] * m_LP[7] - m_LP[3] * m_LP[5]) * m_RP[6] + \
		  (m_LP[3] * m_LP[6] - m_LP[2] * m_LP[7]) * m_RP[5]) * m_RP[8] + \
		 (m_LP[1] * m_LP[4] - m_LP[0] * m_LP[5]) * m_RP[10] * m_RP[7] + \
		 (m_LP[0] * m_LP[5] - m_LP[1] * m_LP[4]) * m_RP[11] * m_RP[6] + \
		 ((m_LP[2] * m_LP[4] - m_LP[0] * m_LP[6]) * m_RP[11] + \
		  (m_LP[0] * m_LP[7] - m_LP[3] * m_LP[4]) * m_RP[10]) * m_RP[5] + \
		 ((m_LP[1] * m_LP[6] - m_LP[2] * m_LP[5]) * m_RP[11] + \
		  (m_LP[3] * m_LP[5] - m_LP[1] * m_LP[7]) * m_RP[10]) * m_RP[4])

#define EQ2 (((m_LP[2] * m_LP[4] - m_LP[0] * m_LP[6]) * m_RP[3] + \
		  (m_LP[0] * m_LP[7] - m_LP[3] * m_LP[4]) * m_RP[2] + \
		  (m_LP[3] * m_LP[6] - m_LP[2] * m_LP[7]) * m_RP[0]) * m_RP[9] + \
		 ((m_LP[1] * m_LP[6] - m_LP[2] * m_LP[5]) * m_RP[3] + \
		  (m_LP[3] * m_LP[5] - m_LP[1] * m_LP[7]) * m_RP[2] + \
		  (m_LP[2] * m_LP[7] - m_LP[3] * m_LP[6]) * m_RP[1]) * m_RP[8] + \
		 (m_LP[0] * m_LP[5] - m_LP[1] * m_LP[4]) * m_RP[10] * m_RP[3] + \
		 (m_LP[1] * m_LP[4] - m_LP[0] * m_LP[5]) * m_RP[11] * m_RP[2] + \
		 ((m_LP[0] * m_LP[6] - m_LP[2] * m_LP[4]) * m_RP[1] + \
		  (m_LP[2] * m_LP[5] - m_LP[1] * m_LP[6]) * m_RP[0]) * m_RP[11] + \
		 ((m_LP[3] * m_LP[4] - m_LP[0] * m_LP[7]) * m_RP[1] + \
		  (m_LP[1] * m_LP[7] - m_LP[3] * m_LP[5]) * m_RP[0]) * m_RP[10])

#define ER2 (((m_LP[0] * m_LP[5] - m_LP[1] * m_LP[4]) * m_RP[2] + \
		  (m_LP[2] * m_LP[4] - m_LP[0] * m_LP[6]) * m_RP[1] + \
		  (m_LP[1] * m_LP[6] - m_LP[2] * m_LP[5]) * m_RP[0]) * m_RP[7] + \
		 ((m_LP[1] * m_LP[4] - m_LP[0] * m_LP[5]) * m_RP[3] + \
		  (m_LP[0] * m_LP[7] - m_LP[3] * m_LP[4]) * m_RP[1] + \
		  (m_LP[3] * m_LP[5] - m_LP[1] * m_LP[7]) * m_RP[0]) * m_RP[6] + \
		 ((m_LP[0] * m_LP[6] - m_LP[2] * m_LP[4]) * m_RP[3] + \
		  (m_LP[3] * m_LP[4] - m_LP[0] * m_LP[7]) * m_RP[2] + \
		  (m_LP[2] * m_LP[7] - m_LP[3] * m_LP[6]) * m_RP[0]) * m_RP[5] + \
		 ((m_LP[2] * m_LP[5] - m_LP[1] * m_LP[6]) * m_RP[3] + \
		  (m_LP[1] * m_LP[7] - m_LP[3] * m_LP[5]) * m_RP[2] + \
		  (m_LP[3] * m_LP[6] - m_LP[2] * m_LP[7]) * m_RP[1]) * m_RP[4])

#define EP3 (((m_LP[0] * m_LP[10] - m_LP[2] * m_LP[8]) * m_RP[7] + \
		  (m_LP[3] * m_LP[8] - m_LP[0] * m_LP[11]) * m_RP[6] + \
		  (m_LP[11] * m_LP[2] - m_LP[10] * m_LP[3]) * m_RP[4]) * m_RP[9] + \
		 ((m_LP[2] * m_LP[9] - m_LP[1] * m_LP[10]) * m_RP[7] + \
		  (m_LP[1] * m_LP[11] - m_LP[3] * m_LP[9]) * m_RP[6] + \
		  (m_LP[10] * m_LP[3] - m_LP[11] * m_LP[2]) * m_RP[5]) * m_RP[8] + \
		 (m_LP[1] * m_LP[8] - m_LP[0] * m_LP[9]) * m_RP[10] * m_RP[7] + \
		 (m_LP[0] * m_LP[9] - m_LP[1] * m_LP[8]) * m_RP[11] * m_RP[6] + \
		 ((m_LP[2] * m_LP[8] - m_LP[0] * m_LP[10]) * m_RP[11] + \
		  (m_LP[0] * m_LP[11] - m_LP[3] * m_LP[8]) * m_RP[10]) * m_RP[5] + \
		 ((m_LP[1] * m_LP[10] - m_LP[2] * m_LP[9]) * m_RP[11] + \
		  (m_LP[3] * m_LP[9] - m_LP[1] * m_LP[11]) * m_RP[10]) * m_RP[4])

#define EQ3 (((m_LP[2] * m_LP[8] - m_LP[0] * m_LP[10]) * m_RP[3] + \
		  (m_LP[0] * m_LP[11] - m_LP[3] * m_LP[8]) * m_RP[2] + \
		  (m_LP[10] * m_LP[3] - m_LP[11] * m_LP[2]) * m_RP[0]) * m_RP[9] + \
		 ((m_LP[1] * m_LP[10] - m_LP[2] * m_LP[9]) * m_RP[3] + \
		  (m_LP[3] * m_LP[9] - m_LP[1] * m_LP[11]) * m_RP[2] + \
		  (m_LP[11] * m_LP[2] - m_LP[10] * m_LP[3]) * m_RP[1]) * m_RP[8] + \
		 (m_LP[0] * m_LP[9] - m_LP[1] * m_LP[8]) * m_RP[10] * m_RP[3] + \
		 (m_LP[1] * m_LP[8] - m_LP[0] * m_LP[9]) * m_RP[11] * m_RP[2] + \
		 ((m_LP[0] * m_LP[10] - m_LP[2] * m_LP[8]) * m_RP[1] + \
		  (m_LP[2] * m_LP[9] - m_LP[1] * m_LP[10]) * m_RP[0]) * m_RP[11] + \
		 ((m_LP[3] * m_LP[8] - m_LP[0] * m_LP[11]) * m_RP[1] + \
		  (m_LP[1] * m_LP[11] - m_LP[3] * m_LP[9]) * m_RP[0]) * m_RP[10])

#define ER3 (((m_LP[0] * m_LP[9] - m_LP[1] * m_LP[8]) * m_RP[2] + \
		  (m_LP[2] * m_LP[8] - m_LP[0] * m_LP[10]) * m_RP[1] + \
		  (m_LP[1] * m_LP[10] - m_LP[2] * m_LP[9]) * m_RP[0]) * m_RP[7] + \
		 ((m_LP[1] * m_LP[8] - m_LP[0] * m_LP[9]) * m_RP[3] + \
		  (m_LP[0] * m_LP[11] - m_LP[3] * m_LP[8]) * m_RP[1] + \
		  (m_LP[3] * m_LP[9] - m_LP[1] * m_LP[11]) * m_RP[0]) * m_RP[6] + \
		 ((m_LP[0] * m_LP[10] - m_LP[2] * m_LP[8]) * m_RP[3] + \
		  (m_LP[3] * m_LP[8] - m_LP[0] * m_LP[11]) * m_RP[2] + \
		  (m_LP[11] * m_LP[2] - m_LP[10] * m_LP[3]) * m_RP[0]) * m_RP[5] + \
		 ((m_LP[2] * m_LP[9] - m_LP[1] * m_LP[10]) * m_RP[3] + \
		  (m_LP[1] * m_LP[11] - m_LP[3] * m_LP[9]) * m_RP[2] + \
		  (m_LP[10] * m_LP[3] - m_LP[11] * m_LP[2]) * m_RP[1]) * m_RP[4])

#define	DET_LURU ((m_LP[10] * m_RP[4] - m_LP[8] * m_RP[6]) * m_RP[9] + \
		(m_LP[9] * m_RP[6] - m_LP[10] * m_RP[5]) * m_RP[8] + \
		(m_LP[8] * m_RP[5] - m_LP[9] * m_RP[4]) * m_RP[10])

#define	DET_LURV ((m_LP[8] * m_RP[2] - m_LP[10] * m_RP[0]) * m_RP[9] + \
		(m_LP[10] * m_RP[1] - m_LP[9] * m_RP[2]) * m_RP[8] + \
		(m_LP[9] * m_RP[0] - m_LP[8] * m_RP[1]) * m_RP[10])

#define DET_LU	((m_LP[8] * m_RP[1] - m_LP[9] * m_RP[0]) * m_RP[6] + \
		(m_LP[10] * m_RP[0] - m_LP[8] * m_RP[2]) * m_RP[5] + \
		(m_LP[9] * m_RP[2] - m_LP[10] * m_RP[1]) * m_RP[4])

#define DET_RU	((m_LP[0] * m_RP[6] - m_LP[2] * m_RP[4]) * m_RP[9] + \
		(m_LP[2] * m_RP[5] - m_LP[1] * m_RP[6]) * m_RP[8] + \
		(m_LP[1] * m_RP[4] - m_LP[0] * m_RP[5]) * m_RP[10])

#define DET_RV	((m_LP[2] * m_RP[0] - m_LP[0] * m_RP[2]) * m_RP[9] + \
		(m_LP[1] * m_RP[2] - m_LP[2] * m_RP[1]) * m_RP[8] + \
		(m_LP[0] * m_RP[1] - m_LP[1] * m_RP[0]) * m_RP[10])

#define DET_C	((m_LP[1] * m_RP[0] - m_LP[0] * m_RP[1]) * m_RP[6] + \
		(m_LP[0] * m_RP[2] - m_LP[2] * m_RP[0]) * m_RP[5] + \
		(m_LP[2] * m_RP[1] - m_LP[1] * m_RP[2]) * m_RP[4])

#define	IR11_LURV (m_LP[10] * m_RP[9] - m_LP[9] * m_RP[10])
#define IR11_LU	(m_LP[9] * m_RP[6] - m_LP[10] * m_RP[5])
#define	IR11_RV	(m_LP[1] * m_RP[10] - m_LP[2] * m_RP[9])
#define IR11_C	(m_LP[2] * m_RP[5] - m_LP[1] * m_RP[6])

#define	IR12_LURU (m_LP[9] * m_RP[10] - m_LP[10] * m_RP[9])
#define IR12_LU	(m_LP[10] * m_RP[1] - m_LP[9] * m_RP[2])
#define	IR12_RU	(m_LP[2] * m_RP[9] - m_LP[1] * m_RP[10])
#define IR12_C	(m_LP[1] * m_RP[2] - m_LP[2] * m_RP[1])

#define	IR13_RU	(m_RP[10] * m_RP[5] - m_RP[6] * m_RP[9])
#define	IR13_RV (m_RP[2] * m_RP[9] - m_RP[1] * m_RP[10])
#define IR13_C	(m_RP[1] * m_RP[6] - m_RP[2] * m_RP[5])

#define	IR21_LURV (m_LP[8] * m_RP[10] - m_LP[10] * m_RP[8])
#define IR21_LU	(m_LP[10] * m_RP[4] - m_LP[8] * m_RP[6])
#define	IR21_RV	(m_LP[2] * m_RP[8] - m_LP[0] * m_RP[10])
#define IR21_C	(m_LP[0] * m_RP[6] - m_LP[2] * m_RP[4])

#define	IR22_LURU (m_LP[10] * m_RP[8] - m_LP[8] * m_RP[10])
#define IR22_LU	(m_LP[8] * m_RP[2] - m_LP[10] * m_RP[0])
#define	IR22_RU	(m_LP[0] * m_RP[10] - m_LP[2] * m_RP[8])
#define IR22_C	(m_LP[2] * m_RP[0] - m_LP[0] * m_RP[2])

#define	IR23_RU	(m_RP[6] * m_RP[8] - m_RP[10] * m_RP[4])
#define	IR23_RV	(m_RP[0] * m_RP[10] - m_RP[2] * m_RP[8])
#define IR23_C	(m_RP[2] * m_RP[4] - m_RP[0] * m_RP[6])

#define	IR31_LURV (m_LP[9] * m_RP[8] - m_LP[8] * m_RP[9])
#define IR31_LU	(m_LP[8] * m_RP[5] - m_LP[9] * m_RP[4])
#define	IR31_RV	(m_LP[0] * m_RP[9] - m_LP[1] * m_RP[8])
#define IR31_C	(m_LP[1] * m_RP[4] - m_LP[0] * m_RP[5])

#define	IR32_LURU (m_LP[8] * m_RP[9] - m_LP[9] * m_RP[8])
#define IR32_LU	(m_LP[9] * m_RP[0] - m_LP[8] * m_RP[1])
#define	IR32_RU	(m_LP[1] * m_RP[8] - m_LP[0] * m_RP[9])
#define IR32_C	(m_LP[0] * m_RP[1] - m_LP[1] * m_RP[0])

#define	IR33_RU	(m_RP[4] * m_RP[9] - m_RP[5] * m_RP[8])
#define	IR33_RV	(m_RP[1] * m_RP[8] - m_RP[0] * m_RP[9])
#define IR33_C	(m_RP[0] * m_RP[5] - m_RP[1] * m_RP[4])

typedef struct {
	int h0, v0;
	int sh1, sv1;
	float	fval;
	int check;
	int matched_h;
	int matched_v;
	float pos[3];
} SM3D_data_t2;

struct DRAWRECT {
	int h0;
	int v0;
	int h1;
	int v1;
};

enum VisionMode {OPTICALFLOW, STEREO};

using namespace std;
namespace VisionRTM
{
    class OpticalAndStereo: public vision_rtm::VideoFrameSyncComponentBase
    {
    public:
        OpticalAndStereo(RTC::Manager* manager);
        virtual ~OpticalAndStereo();

    protected:
        virtual RTC::ReturnCode_t onInitialize();
        virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);
        virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);
        virtual RTC::ReturnCode_t onFinalize();
        virtual RTC::ReturnCode_t onChangeFrame(ImageProcessingAPI::VideoMemory& captured_image);
        void initializeVideo(VisionMode vm);
        void initStereoParams (void);
        void CalcEpiParams_L (int ru, int rv);
        void CalcEpiParams_R (int lu, int lv);
        int CalcEpiLv (int lu);
        int CalcEpiRv (int ru);
        void CalcPos (int ru, int rv, int lu, float pos[3]);

    private:
        RTC::TimedShort m_Mode;
        RTC::InPort<RTC::TimedShort> m_ParamIn;
        
        RTC::TimedFloatSeq m_Result;
        RTC::OutPort<RTC::TimedFloatSeq> m_ResultOut;
        
        VisionMode vmode; //mode 0: optical flow, mode 1: 3d stereo

        int f0, f1;
        int 	dColor;
        unsigned int 	time;
        Flow_data_t	*flowdata, *pF;
        int 	dist[1];
        char 	str[256];

        float	m_Dluru, m_Dlurv;
        float	m_Dlu, m_Dru, m_Drv, m_Dc;
        float	m_R11lurv, m_R11lu, m_R11rv, m_R11c;
        float	m_R12luru, m_R12lu, m_R12ru, m_R12c;
        float	m_R13ru, m_R13rv, m_R13c;
        float	m_R21lurv, m_R21lu, m_R21rv, m_R21c;
        float	m_R22luru, m_R22lu, m_R22ru, m_R22c;
        float	m_R23ru, m_R23rv, m_R23c;
        float	m_R31lurv, m_R31lu, m_R31rv, m_R31c;
        float	m_R32luru, m_R32lu, m_R32ru, m_R32c;
        float	m_R33ru, m_R33rv, m_R33c;
        float	m_EP1, m_EP2, m_EP3;
        float	m_EQ1, m_EQ2, m_EQ3;
        float	m_ER1, m_ER2, m_ER3;
        float	m_LA1, m_LA2, m_LA3;
        float	m_RA1, m_RA2, m_RA3;

        int start_h;
        int start_v;
        int size_h;
        int size_v;
        SM3D_data_t2 *feature;
        SM3D_data_t2 *feature_points;
        int st0_shsize;
        int st0_svsize;
        int st0_offset;
        int pitch_h;
        int pitch_v;
        float c_thr;
        float l_thr;
        float match_th;
        int smem_r;
        int smem_l;
  
        int sobelH_mem;
        int sobelH_h0;
        int sobelH_v0;
        int sobelV_mem;
        int sobelV_h0;
        int sobelV_v0;
        
        int fpoints, mpoints;
        int dh, dv;
        float fval;
        int match_dh, match_dv;
        float match_dist;

        int drawNum;
        DRAWRECT drawRect[8192];
        /** @brief  Configurationのエラーチェックをおこなう*/
        //virtual bool hasValidConfiguration();
    };
} // end of namespace VisionRTM

extern "C"
{
    void OpticalAndStereoInit(RTC::Manager* manager);
}

#endif // CORNER_H_INCLUDED_
